@@ -1,6 +1,5 @@
 #!/bin/bash
 
 # Calls pyspread from top level folder of extracted tarball
-
-export PYTHONPATH=$PYTHONPATH:./pyspread
-./pyspread/pyspread $@
+export PYTHONPATH=$PYTHONPATH:.
+python3 ./src/pyspread.py $@
