@@ -113,13 +113,13 @@ class MainWindowActions(AttrDict):
                               statustip='Save spreadsheet to a new file')
 
         self.imprt = Action(self.parent, "&Import",
-                            self.parent.on_nothing,
+                            self.parent.workflows.file_import,
                             icon=Icon.imprt,
                             statustip='Import a file and paste it into the '
                                       'current grid')
 
         self.export = Action(self.parent, "&Export",
-                             self.parent.on_nothing,
+                             self.parent.workflows.file_export,
                              icon=Icon.export,
                              statustip="Export selection to a file")
 
@@ -483,6 +483,13 @@ class MainWindowActions(AttrDict):
                                 statustip='Lock cell so that its code '
                                           'cannot be changed')
 
+        self.button_cell = Action(self.parent, "Button cell",
+                                  self.parent.grid.on_button_cell_pressed,
+                                  icon=Icon.button,
+                                  checkable=True,
+                                  statustip='Make cell a button cell that is '
+                                            'executed only when pressed')
+
         self.merge_cells = Action(self.parent, "Merge cells",
                                   self.parent.grid.on_merge_pressed,
                                   icon=Icon.merge_cells,
@@ -715,23 +722,17 @@ class MainWindowActions(AttrDict):
     def create_help_actions(self):
         """actions for Help menu"""
 
-        self.first_steps = Action(self.parent, "First steps...",
-                                  self.parent.on_nothing,
-                                  icon=Icon.help,
-                                  shortcut='F1',
-                                  statustip='Display the first steps document '
-                                            'with an overview of pyspread')
+        self.manual = Action(self.parent, "Manual...",
+                             self.parent.on_manual,
+                             icon=Icon.help,
+                             shortcut='F1',
+                             statustip='Display the pyspread manual')
 
         self.tutorial = Action(self.parent, "Tutorial...",
-                               self.parent.on_nothing,
+                               self.parent.on_tutorial,
                                icon=Icon.tutorial,
                                statustip='Display a pyspread tutorial')
 
-        self.faq = Action(self.parent, "FAQ...",
-                          self.parent.on_nothing,
-                          icon=Icon.faq,
-                          statustip='Display frequently asked questions')
-
         self.dependencies = Action(self.parent, "Dependencies...",
                                    self.parent.on_dependencies,
                                    icon=Icon.dependencies,
