@@ -38,6 +38,7 @@ import numpy
 from PyQt5.QtWidgets import QTableView, QStyledItemDelegate, QTabBar, QWidget
 from PyQt5.QtWidgets import QStyleOptionViewItem, QApplication, QStyle
 from PyQt5.QtWidgets import QAbstractItemDelegate, QHeaderView, QFontDialog
+from PyQt5.QtWidgets import QInputDialog, QLineEdit
 from PyQt5.QtGui import QColor, QBrush, QPen, QFont, QPainter, QPalette
 from PyQt5.QtGui import QImage as BasicQImage
 from PyQt5.QtGui import QAbstractTextDocumentLayout, QTextDocument
@@ -57,6 +58,7 @@ from src.commands import CommandInsertTable, CommandDeleteTable
 from src.commands import CommandDeleteColumns, CommandSetCellMerge
 from src.commands import CommandSetCellRenderer, CommandSetRowsHeight
 from src.commands import CommandSetColumnsWidth, CommandSetCellTextAlignment
+from src.commands import CommandMakeButtonCell, CommandRemoveButtonCell
 from src.model.model import CodeArray
 from src.lib.selection import Selection
 from src.lib.string_helpers import quote, wrap_text, get_svg_aspect
@@ -438,11 +440,7 @@ class Grid(QTableView):
 
         Does neither emit dataChanged nor clear _attr_cache or _table_cache.
 
-        Parameters
-        ----------
-
-        * key: 3-tuple of int
-        \trow, column, table tuple
+        :key: 3-tuple of int: row, column, table
 
         """
 
@@ -891,6 +889,35 @@ class Grid(QTableView):
             command = CommandThawCell(self.model, self.current, description)
         self.main_window.undo_stack.push(command)
 
+    def on_button_cell_pressed(self, toggled):
+        """Button cell event handler"""
+
+        current_attr = self.model.code_array.cell_attributes[self.current]
+        if not toggled and current_attr["button_cell"] is False \
+           or toggled and current_attr["button_cell"] is not False:
+            # Something is not syncronized in the menu
+            return
+
+        if toggled:
+            # Get button text from user
+            text, accept = QInputDialog.getText(self.main_window,
+                                                "Make button cell",
+                                                "Button text:",
+                                                QLineEdit.Normal, "")
+            if accept and text:
+                description_tpl = "Make cell {} a button cell"
+                description = description_tpl.format(self.current)
+                command = CommandMakeButtonCell(self, text,
+                                                self.currentIndex(),
+                                                description)
+                self.main_window.undo_stack.push(command)
+        else:
+            description_tpl = "Make cell {} a non-button cell"
+            description = description_tpl.format(self.current)
+            command = CommandRemoveButtonCell(self, self.currentIndex(),
+                                              description)
+            self.main_window.undo_stack.push(command)
+
     def on_merge_pressed(self):
         """Merge cells button pressed event handler"""
 
@@ -1717,7 +1744,7 @@ class GridCellDelegate(QStyledItemDelegate):
     def createEditor(self, parent, option, index):
         """Overloads QStyledItemDelegate
 
-        Disables editor in frozen cells
+        Disables editor in locked cells
         Switches to chart dialog in chart cells
 
         """
@@ -1748,8 +1775,12 @@ class GridCellDelegate(QStyledItemDelegate):
            and source is self.editor \
            and event.modifiers() == Qt.ControlModifier \
            and event.key() in (Qt.Key_Return, Qt.Key_Enter):
-            self.grid.model.setData(self.grid.currentIndex(),
-                                    quote(source.text()), Qt.EditRole)
+
+            code = quote(source.text())
+            index = self.grid.currentIndex()
+            description = "Quote code for cell {}".format(index)
+            cmd = CommandSetCellCode(code, self.grid.model, index, description)
+            self.main_window.undo_stack.push(cmd)
         return QWidget.eventFilter(self, source, event)
 
     def setEditorData(self, editor, index):
