@@ -21,12 +21,15 @@
 
 """
 
+
+pyspread
+========
+
 - Main Python spreadsheet application
 - Run this script to start the application.
 
 **Provides**
 
-* Commandlineparser: Gets command line options and parameters
 * MainApplication: Initial command line operations and application launch
 * :class:`MainWindow`: Main windows class
 
@@ -44,6 +47,7 @@ from PyQt5.QtGui import QColor, QFont, QPalette, QPainter
 from PyQt5.QtPrintSupport import QPrinter, QPrintDialog, QPrintPreviewDialog
 
 from src import VERSION, APP_NAME
+from src.cli import ArgumentParser
 from src.settings import Settings
 from src.icons import Icon, IconPath
 from src.grid import Grid
@@ -53,8 +57,8 @@ from src.toolbar import MainToolBar, FindToolbar, FormatToolbar, MacroToolbar
 from src.actions import MainWindowActions
 from src.workflows import Workflows
 from src.widgets import Widgets
-from src.dialogs import ApproveWarningDialog, PreferencesDialog
-from src.dialogs import PrintAreaDialog
+from src.dialogs import ApproveWarningDialog, PreferencesDialog, ManualDialog
+from src.dialogs import TutorialDialog, PrintAreaDialog
 from src.installer import DependenciesDialog
 from src.panels import MacroPanel
 from src.lib.hashing import genkey
@@ -70,6 +74,8 @@ QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
 class MainWindow(QMainWindow):
     """Pyspread main window
 
+    :application: QApplication
+    :args: Command line arguments object from argparse
     :unit_test: If True then the application runs in unit_test mode
     :type unit_test: bool, defaults to False
 
@@ -77,7 +83,7 @@ class MainWindow(QMainWindow):
 
     gui_update = pyqtSignal(dict)
 
-    def __init__(self, application, unit_test=False):
+    def __init__(self, application, args, unit_test=False):
         super().__init__()
 
         self._loading = True
@@ -116,6 +122,11 @@ class MainWindow(QMainWindow):
         self._loading = False
         self._previous_window_state = self.windowState()
 
+        # Open initial file if provided by the command line
+        if args.file is not None:
+            self.workflows.filepath_open(args.file)
+            self.workflows.update_main_window_title()
+
     def _init_window(self):
         """Initialize main window components"""
 
@@ -295,66 +306,36 @@ class MainWindow(QMainWindow):
         painter = QPainter(printer)
         option = QStyleOptionViewItem()
 
-        painter.setViewport(self.grid.rect())
-        painter.setWindow(self.grid.rect())
+        with self.workflows.standard_zoom():
+            painter.setViewport(self.grid.rect())
+            painter.setWindow(self.grid.rect())
 
-        top, left, bottom, right = self.print_area
+            rows = self.workflows.get_paint_rows(self.print_area)
+            columns = self.workflows.get_paint_columns(self.print_area)
+            total_height = self.workflows.get_total_height(self.print_area)
+            total_width = self.workflows.get_total_width(self.print_area)
 
-        rows = range(top, bottom + 1)
-        columns = range(left, right + 1)
+            area = printer.paperRect()
+            left, top, right, bottom = \
+                printer.getPageMargins(QPrinter.DevicePixel)
 
-        total_width = 0
-        total_height = 0
+            clip_rect = QRect(area.x()+left, area.y()+top,
+                              area.width()-left-right,
+                              area.height()-top-bottom)
+            painter.setClipRect(clip_rect)
 
-        for row in rows:
-            total_height += self.grid.rowHeight(row)
-        for column in columns:
-            total_width += self.grid.columnWidth(column)
+            xscale = (area.width() - 2*left - 2*right) / total_width
+            yscale = (area.height() - 2*top - 2*bottom) / total_height
 
-        area = printer.paperRect()
-        left, top, right, bottom = printer.getPageMargins(QPrinter.DevicePixel)
+            scale = min(xscale, yscale)
+            painter.save()
 
-        clip_rect = QRect(area.x()+left, area.y()+top,
-                          area.width()-left-right, area.height()-top-bottom)
-        painter.setClipRect(clip_rect)
+            painter.scale(scale, scale)
+            painter.translate((-area.x() + left) / scale,
+                              (-area.y() + top) / scale)
 
-        xscale = (area.width() - 2*left - 2*right) / total_width
-        yscale = (area.height() - 2*top - 2*bottom) / total_height
-
-        scale = min(xscale, yscale)
-        painter.save()
-
-        painter.scale(scale, scale)
-        painter.translate((-area.x() + left) / scale,
-                          (-area.y() + top) / scale)
-
-        x_offset = self.grid.columnViewportPosition(0)
-        y_offset = self.grid.rowViewportPosition(0)
-
-        code_array = self.grid.model.code_array
-        cell_attributes = code_array.cell_attributes
-
-        for row in rows:
-            for column in columns:
-                idx = self.grid.model.index(row, column)
-                visual_rect = self.grid.visualRect(idx)
-                option.rect = QRect(visual_rect.x() - x_offset,
-                                    visual_rect.y() - y_offset,
-                                    visual_rect.width(), visual_rect.height())
-
-                key = row, column, self.grid.table
-                option.text = code_array(key)
-                option.widget = self.grid
-
-                merging_cell = cell_attributes.get_merging_cell(key)
-                if merging_cell is None \
-                   or merging_cell[0] == row and merging_cell[1] == column:
-                    painter.setClipRect(option.rect)
-                    self.grid.itemDelegate().paint(painter, option, idx)
-
-        painter.restore()
-
-        painter.end()
+            self.workflows.paint(painter, option, clip_rect, rows, columns)
+            painter.restore()
 
     def on_nothing(self):
         """Dummy action that does nothing"""
@@ -484,6 +465,18 @@ class MainWindow(QMainWindow):
 
         self._toggle_widget(self.macro_dock, "toggle_macro_panel", toggled)
 
+    def on_manual(self):
+        """Show manual browser"""
+
+        dialog = ManualDialog(self)
+        dialog.show()
+
+    def on_tutorial(self):
+        """Show tutorial browser"""
+
+        dialog = TutorialDialog(self)
+        dialog.show()
+
     def on_about(self):
         """Show about message box"""
 
@@ -541,6 +534,9 @@ class MainWindow(QMainWindow):
         lock_action.setChecked(attributes["locked"])
         self.entry_line.setReadOnly(attributes["locked"])
 
+        button_action = self.main_window_actions.button_cell
+        button_action.setChecked(attributes["button_cell"] is not False)
+
         rotation = "rotate_{angle}".format(angle=int(attributes["angle"]))
         widgets.rotate_button.set_current_action(rotation)
         widgets.rotate_button.set_menu_checked(rotation)
@@ -585,8 +581,11 @@ class MainWindow(QMainWindow):
 
 
 def main():
+    parser = ArgumentParser()
+    args = parser.parse_args()
+
     app = QApplication(sys.argv)
-    main_window = MainWindow(app)
+    main_window = MainWindow(app, args)
 
     app.exec_()
 
