@@ -29,19 +29,21 @@ from base64 import b85encode
 import bz2
 from contextlib import contextmanager
 from copy import deepcopy
+import csv
 from itertools import cycle
 import io
+from itertools import takewhile, repeat
 import os.path
 from pathlib import Path
 from shutil import move
 import sys
 from tempfile import NamedTemporaryFile
 
-from PyQt5.QtCore import Qt, QMimeData, QModelIndex, QBuffer
-from PyQt5.QtGui import QImage as BasicQImage
-from PyQt5.QtGui import QTextDocument, QImage
+from PyQt5.QtCore import Qt, QMimeData, QModelIndex, QBuffer, QRect, QSize
+from PyQt5.QtGui import QTextDocument, QImage, QPainter
 from PyQt5.QtWidgets import QApplication, QProgressDialog, QMessageBox
-from PyQt5.QtWidgets import QInputDialog
+from PyQt5.QtWidgets import QInputDialog, QStyleOptionViewItem
+from PyQt5.QtSvg import QSvgGenerator
 
 try:
     import matplotlib.figure as matplotlib_figure
@@ -53,10 +55,14 @@ from src.commands import CommandSetGridSize
 from src.dialogs import DiscardChangesDialog, FileOpenDialog, GridShapeDialog
 from src.dialogs import FileSaveDialog, ImageFileOpenDialog, ChartDialog
 from src.dialogs import CellKeyDialog, FindDialog, ReplaceDialog
+from src.dialogs import CsvFileImportDialog, CsvImportDialog, CsvExportDialog
+from src.dialogs import CsvExportAreaDialog, CsvFileExportDialog
+from src.dialogs import SvgExportAreaDialog
 from src.interfaces.pys import PysReader, PysWriter
 from src.lib.hashing import sign, verify
 from src.lib.selection import Selection
 from src.lib.typechecks import is_svg
+from src.lib.csv import csv_reader, convert
 
 
 class Workflows:
@@ -64,7 +70,7 @@ class Workflows:
         self.main_window = main_window
 
     @contextmanager
-    def progress_dialog(self, title, label, maximum, min_duration=3000):
+    def progress_dialog(self, title, label, maximum):
         """:class:`~contextlib.contextmanager` that displays a progress dialog
         """
 
@@ -73,14 +79,11 @@ class Workflows:
         progress_dialog.setWindowModality(Qt.WindowModal)
         progress_dialog.setLabelText(label)
         progress_dialog.setMaximum(maximum)
-        progress_dialog.setMinimumDuration(min_duration)
-        if not self.main_window.unit_test:
-            progress_dialog.show()
-        progress_dialog.setValue(0)
 
         yield progress_dialog
 
         progress_dialog.setValue(maximum)
+        progress_dialog.close()
 
     @contextmanager
     def disable_entryline_updates(self):
@@ -107,7 +110,8 @@ class Workflows:
           is executed.
 
         If no changes are present then `func` is directly executed.
-        After executing `func`, :func:`reset_changed_since_save` is called.
+        After executing `func`, :func:`reset_changed_since_save` and
+        `update_main_window_title` are called.
 
         """
 
@@ -128,6 +132,7 @@ class Workflows:
             except TypeError:
                 func(self)  # No args accepted
             self.reset_changed_since_save()
+            self.update_main_window_title()
 
         return function_wrapper
 
@@ -137,10 +142,12 @@ class Workflows:
         # Change the main window filepath state
         self.main_window.settings.changed_since_save = False
 
+    def update_main_window_title(self):
+        """Change the main window title to reflect the current file name"""
+
         # Get the current filepath
         filepath = self.main_window.settings.last_file_input_path
 
-        # Change the main window title
         window_title = "{filename} - pyspread".format(filename=filepath.name)
         self.main_window.setWindowTitle(window_title)
 
@@ -165,8 +172,6 @@ class Workflows:
             self.main_window.statusBar().showMessage(msg)
             return
 
-        self.main_window.grid.current = 0, 0, 0
-
         # Reset grid
         self.main_window.grid.model.reset()
 
@@ -176,13 +181,15 @@ class Workflows:
         # Select upper left cell because initial selection behaves strange
         self.main_window.grid.reset_selection()
 
+        # Set current cell to upper left corner
+        self.main_window.grid.current = 0, 0, 0
+
         # Exit safe mode
         self.main_window.safe_mode = False
 
-    def _filepath_open(self, filepath):
-        """Workflow for opening a file if a filepath is known"""
+    def _get_filesize(self, filepath):
+        """Returns the filesize"""
 
-        code_array = self.main_window.grid.model.code_array
         try:
             filesize = os.path.getsize(filepath)
         except OSError as err:
@@ -190,6 +197,16 @@ class Workflows:
             msg = msg_tpl.format(filepath=filepath, err=err)
             self.main_window.statusBar().showMessage(msg)
             return
+        return filesize
+
+    def filepath_open(self, filepath):
+        """Workflow for opening a file if a filepath is known"""
+
+        code_array = self.main_window.grid.model.code_array
+
+        filesize = self._get_filesize(filepath)
+        if filesize is None:
+            return
 
         # Reset grid
         self.main_window.grid.model.reset()
@@ -279,13 +296,13 @@ class Workflows:
                 return  # Cancel pressed
             filepath = Path(dial.file_path).with_suffix(dial.suffix)
 
-        self._filepath_open(filepath)
+        self.filepath_open(filepath)
 
     @handle_changed_since_save
     def file_open_recent(self, filepath):
         """File open recent workflow"""
 
-        self._filepath_open(Path(filepath))
+        self.filepath_open(Path(filepath))
 
     def sign_file(self, filepath):
         """Signs filepath if not in :attr:`model.model.DataArray.safe_mode`"""
@@ -411,6 +428,248 @@ class Workflows:
 
         self._save(fp)
 
+    def file_import(self):
+        """Import csv files"""
+
+        def rawincount(filepath):
+            """Counts lines of file"""
+
+            with open(filepath, 'rb') as f:
+                bufgen = takewhile(lambda x: x, (f.raw.read(1024*1024)
+                                                 for _ in repeat(None)))
+                return sum(buf.count(b'\n') for buf in bufgen)
+
+        if self.main_window.unit_test:
+            # We are in a unit test and use the unit test filepath
+            filepath = self.main_window.unit_test_data
+
+        else:
+            # Get filepath from user
+            dial = CsvFileImportDialog(self.main_window)
+            if not dial.file_path:
+                return  # Cancel pressed
+            filepath = Path(dial.file_path)
+
+        csv_dlg = CsvImportDialog(self.main_window, filepath)
+
+        if not csv_dlg.exec():
+            return
+
+        # Dialog accepted, now fill the grid
+
+        row, column, table = current = self.main_window.grid.current
+        model = self.main_window.grid.model
+        rows, columns, tables = model.shape
+
+        description_tpl = "Import from csv file {} at cell {}"
+        description = description_tpl.format(filepath, current)
+
+        try:
+            filelines = rawincount(filepath)
+        except OSError as error:
+            self.main_window.statusBar().showMessage(str(error))
+            return
+
+        command = None
+
+        try:
+            with open(filepath, newline='') as csvfile:
+                title = "csv import progress"
+                label = "Importing {}...".format(filepath.name)
+                with self.progress_dialog(title, label,
+                                          filelines) as progress_dialog:
+                    try:
+                        # Enter safe mode
+                        self.main_window.safe_mode = True
+
+                        reader = csv_reader(csvfile, csv_dlg.dialect,
+                                            csv_dlg.digest_types)
+                        for i, line in enumerate(reader):
+                            if row + i >= rows:
+                                break
+                            if i % 100 == 0:
+                                progress_dialog.setValue(i)
+                                self.main_window.application.processEvents()
+                                if progress_dialog.wasCanceled():
+                                    return
+
+                            for j, ele in enumerate(line):
+                                if column + j >= columns:
+                                    break
+
+                                if csv_dlg.digest_types is None:
+                                    code = str(ele)
+                                else:
+                                    code = convert(ele,
+                                                   csv_dlg.digest_types[j])
+                                index = model.index(row + i, column + j)
+                                cmd = CommandSetCellCode(code, model, index,
+                                                         description)
+
+                                if command is None:
+                                    command = cmd
+                                else:
+                                    command.mergeWith(cmd)
+                    except ValueError as error:
+                        msg = str(error)
+                        self.main_window.statusBar().showMessage(msg)
+                        return
+
+                    self.main_window.undo_stack.push(command)
+        except OSError as error:
+            self.main_window.statusBar().showMessage(str(error))
+            return
+
+    def file_export(self):
+        """Export csv and svg files"""
+
+        # Get filepath from user
+        dial = CsvFileExportDialog(self.main_window)
+        if not dial.file_path:
+            return  # Cancel pressed
+        filepath = Path(dial.file_path)
+
+        if "CSV" in dial.selected_filter:
+            self._csv_export(filepath)
+        elif "SVG" in dial.selected_filter:
+            # Extend filepath suffix if needed
+            if filepath.suffix != dial.suffix:
+                filepath = filepath.with_suffix(dial.suffix)
+            self._svg_export(filepath)
+
+    def _csv_export(self, filepath):
+        """Export to csv file filepath"""
+
+        # Get area for csv export
+        csv_area = CsvExportAreaDialog(self.main_window,
+                                       self.main_window.grid).area
+        if csv_area is None:
+            return
+
+        top, left, bottom, right = csv_area
+        code_array = self.main_window.grid.model.code_array
+        table = self.main_window.grid.table
+        csv_data = code_array[top: bottom + 1, left: right + 1, table]
+
+        csv_dlg = CsvExportDialog(self.main_window, csv_area)
+
+        if not csv_dlg.exec():
+            return
+
+        try:
+            with open(filepath, "w", newline='') as csvfile:
+                writer = csv.writer(csvfile, dialect=csv_dlg.dialect)
+                writer.writerows(csv_data)
+        except OSError as error:
+            self.main_window.statusBar().showMessage(str(error))
+
+    def _svg_export(self, filepath):
+        """Export to svg file filepath"""
+
+        with self.standard_zoom():
+            grid = self.main_window.grid
+
+            generator = QSvgGenerator()
+            generator.setFileName(str(filepath))
+
+            # Get area for svg export
+            svg_area = SvgExportAreaDialog(self.main_window, grid).area
+            if svg_area is None:
+                return
+
+            rows = self.get_paint_rows(svg_area)
+            columns = self.get_paint_columns(svg_area)
+            total_height = self.get_total_height(svg_area)
+            total_width = self.get_total_width(svg_area)
+
+            generator.setSize(QSize(total_width, total_height))
+            paint_rect = QRect(0, 0, total_width, total_height)
+            generator.setViewBox(paint_rect)
+            option = QStyleOptionViewItem()
+
+            painter = QPainter(generator)
+
+            self.paint(painter, option, paint_rect, rows, columns)
+
+    @contextmanager
+    def standard_zoom(self):
+        """Decorator for tasks that have to take place in standard zoom"""
+
+        zoom = self.main_window.grid.zoom
+        self.main_window.grid.zoom = 1.0
+        yield
+        self.main_window.grid.zoom = zoom
+
+    def get_paint_rows(self, area):
+        """Iterator of rows to paint"""
+
+        rows = self.main_window.grid.model.shape[0]
+        top, _, bottom, _ = area
+        top = max(0, min(rows - 1, top))
+        bottom = max(0, min(rows - 1, bottom))
+
+        return range(top, bottom + 1)
+
+    def get_paint_columns(self, area):
+        """Iterator of columns to paint"""
+
+        columns = self.main_window.grid.model.shape[1]
+        _, left, _, right = area
+        left = max(0, min(columns - 1, left))
+        right = max(0, min(columns - 1, right))
+
+        return range(left, right + 1)
+
+    def get_total_height(self, area):
+        """Total height of paint_rows"""
+
+        grid = self.main_window.grid
+        return sum(grid.rowHeight(row) for row in self.get_paint_rows(area))
+
+    def get_total_width(self, area):
+        """Total height of paint_columns"""
+
+        grid = self.main_window.grid
+        return sum(grid.columnWidth(column)
+                   for column in self.get_paint_columns(area))
+
+    def paint(self, painter, option, paint_rect, rows, columns):
+        """Grid paint workflow for printing and svg export"""
+
+        grid = self.main_window.grid
+        code_array = grid.model.code_array
+        cell_attributes = code_array.cell_attributes
+
+        x_offset = grid.columnViewportPosition(0)
+        y_offset = grid.rowViewportPosition(0)
+
+        for row in rows:
+            for column in columns:
+                key = row, column, grid.table
+                merging_cell = cell_attributes.get_merging_cell(key)
+                if merging_cell is None \
+                   or merging_cell[0] == row and merging_cell[1] == column:
+
+                    idx = grid.model.index(row, column)
+                    visual_rect = grid.visualRect(idx)
+                    x = max(0, visual_rect.x() - x_offset)
+                    y = max(0, visual_rect.y() - y_offset)
+                    width = visual_rect.width()
+                    if visual_rect.x() - x_offset < 0:
+                        width += visual_rect.x() - x_offset
+                    height = visual_rect.height()
+                    if visual_rect.y() - y_offset < 0:
+                        height += visual_rect.y() - y_offset
+
+                    option.rect = QRect(x, y, width, height)
+                    painter.setClipRect(option.rect)
+
+                    option.text = code_array(key)
+                    option.widget = grid
+
+                    grid.itemDelegate().paint(painter, option, idx)
+        painter.end()
+
     @handle_changed_since_save
     def file_quit(self):
         """Program exit workflow"""
@@ -490,7 +749,7 @@ class Workflows:
             clipboard.setText(repr(data))
 
         elif renderer == "image":
-            if isinstance(data, BasicQImage):
+            if isinstance(data, QImage):
                 clipboard.setImage(data)
             else:
                 # We may have an svg image here
